Struct spacetimedb_lib::address::Address
source · pub struct Address(_);Expand description
This is the address for a SpacetimeDB database. It is a unique identifier for a particular database and once set for a database, does not change.
TODO: Evaluate other possible names: DatabaseAddress, SPAddress
TODO: Evaluate replacing this with a literal Ipv6Address which is assigned
permanently to a database.
Implementations
sourceimpl Address
impl Address
pub fn from_arr(arr: &[u8; 16]) -> Self
pub fn from_hex(hex: &str) -> Result<Self, Error>
pub fn to_hex(self) -> String
pub fn to_abbreviated_hex(self) -> String
pub fn from_slice(slice: impl AsRef<[u8]>) -> Self
pub fn as_slice(&self) -> [u8; 16]
pub fn to_ipv6(self) -> Ipv6Addr
pub fn to_ipv6_string(self) -> String