Module spacetimedb_lib::buffer

source ·

Structs

Enums

  • Minimal utility for reading & writing the types we need to internal storage, without relying on third party libraries like bytes::Bytes, etc. Meant to be kept slim and trim for use across both native and wasm.

Traits