Enum spacetimedb_lib::buffer::DecodeError
source · pub enum DecodeError {
BufferLength,
InvalidTag,
InvalidUtf8,
}Expand description
Minimal utility for reading & writing the types we need to internal storage, without relying on third party libraries like bytes::Bytes, etc. Meant to be kept slim and trim for use across both native and wasm.
Variants