FunctionVisibility

Enum FunctionVisibility 

pub enum FunctionVisibility {
    Private,
    ClientCallable,
}
The visibility of a function (reducer or procedure).

Variants



Private

Not callable by arbitrary clients.

Still callable by the module owner, collaborators, and internal module code.

Enabled for lifecycle reducers and scheduled functions by default.



ClientCallable

Callable from client code.