RawModuleDefV10

Struct RawModuleDefV10 

pub struct RawModuleDefV10 {
    pub sections: Vec<RawModuleDefV10Section>,
}
A possibly-invalid raw module definition.

ABI Version 10.

These “raw definitions” may contain invalid data, and are validated by the validate module into a proper spacetimedb_schema::ModuleDef, or a collection of errors.

The module definition maintains the same logical global namespace as V9, mapping Identifiers to:

  • database-level objects:
    • logical schema objects: tables, constraints, sequence definitions
    • physical schema objects: indexes
  • module-level objects: reducers, procedures, schedule definitions
  • binding-level objects: type aliases

All of these types of objects must have unique names within the module. The exception is columns, which need unique names only within a table.

Fields

sections: Vec<RawModuleDefV10Section>

The sections comprising this module definition.

Sections can appear in any order and are optional.

Implementations

Source

impl RawModuleDefV10

pub fn types(&self) -> Option<&Vec<RawTypeDefV10>>

Get the types section, if present.

pub fn tables(&self) -> Option<&Vec<RawTableDefV10>>

Get the tables section, if present.

pub fn typespace(&self) -> Option<&Typespace>

Get the typespace section, if present.

pub fn reducers(&self) -> Option<&Vec<RawReducerDefV10>>

Get the reducers section, if present.

pub fn procedures(&self) -> Option<&Vec<RawProcedureDefV10>>

Get the procedures section, if present.

pub fn views(&self) -> Option<&Vec<RawViewDefV10>>

Get the views section, if present.

pub fn schedules(&self) -> Option<&Vec<RawScheduleDefV10>>

Get the schedules section, if present.

pub fn lifecycle_reducers(&self) -> Option<&Vec<RawLifeCycleReducerDefV10>>

Get the lifecycle reducers section, if present.

pub fn tables_mut_for_tests(&mut self) -> &mut Vec<RawTableDefV10>

pub fn row_level_security(&self) -> Option<&Vec<RawRowLevelSecurityDefV10>>

pub fn case_conversion_policy(&self) -> CaseConversionPolicy

pub fn explicit_names(&self) -> Option<&ExplicitNames>