Type Definition spacetimedb_lib::error::ResultTest
source · pub type ResultTest<T> = Result<T, TestError>;Expand description
A wrapper for using Result in tests, so it display nicely
pub type ResultTest<T> = Result<T, TestError>;A wrapper for using Result in tests, so it display nicely