Struct spacetimedb_lib::relation::Header

source ·
pub struct Header {
    pub table_name: String,
    pub fields: Vec<Column>,
    pub constraints: Vec<(ColList, Constraints)>,
}

Fields

table_name: Stringfields: Vec<Column>constraints: Vec<(ColList, Constraints)>

Implementations

source

impl Header

source

pub fn new( table_name: String, fields: Vec<Column>, constraints: Vec<(ColList, Constraints)> ) -> Header

source

pub fn clone_for_error(&self) -> Header

Equivalent to what Clone::clone would do.

Header intentionally does not implement Clone, as we can’t afford to clone it in normal execution paths. However, we don’t care about performance in error paths, and we need to embed owned Headers in error objects to report useful messages.

source

pub fn from_product_type(table_name: String, fields: ProductType) -> Header

source

pub fn to_product_type(&self) -> ProductType

source

pub fn for_mem_table(fields: ProductType) -> Header

source

pub fn as_without_table_name(&self) -> HeaderOnlyField<'_>

source

pub fn ty(&self) -> ProductType

source

pub fn find_by_name(&self, field_name: &str) -> Option<&Column>

source

pub fn column_pos<'a>(&'a self, col: &'a FieldName) -> Option<ColId>

source

pub fn column_pos_or_err<'a>( &'a self, col: &'a FieldName ) -> Result<ColId, RelationError>

source

pub fn find_pos_by_name(&self, name: &str) -> Option<ColId>

Finds the position of a field with name.

source

pub fn column<'a>(&'a self, col: &'a FieldName) -> Option<&'a Column>

source

pub fn has_constraint(&self, field: &FieldName, constraint: Constraints) -> bool

source

pub fn project( &self, cols: &[impl Into<FieldExpr> + Clone] ) -> Result<Header, RelationError>

Project the FieldExpr & the Constraints that referenced them

source

pub fn extend(&self, right: &Header) -> Header

Adds the fields & Constraints from right to this Header, renaming duplicated fields with a counter like a, a => a, a0.