Struct spacetimedb_lib::ProductValue
source · pub struct ProductValue {
pub elements: Vec<AlgebraicValue, Global>,
}Expand description
A product value is made of a a list of
“elements” / “fields” / “factors” of other AlgebraicValues.
The type of a product value is a product type.
Fields
elements: Vec<AlgebraicValue, Global>The values that make up this product value.
Implementations
sourceimpl ProductValue
impl ProductValue
pub fn decode<'a>( algebraic_type: &<ProductValue as Value>::Type, bytes: &mut impl BufReader<'a> ) -> Result<ProductValue, DecodeError>
pub fn encode(&self, bytes: &mut impl BufWriter)
sourceimpl ProductValue
impl ProductValue
sourcepub fn new(elements: &[AlgebraicValue]) -> ProductValue
pub fn new(elements: &[AlgebraicValue]) -> ProductValue
Returns a product value constructed from the given values in elements.
sourceimpl ProductValue
impl ProductValue
sourcepub fn get_field(
&self,
index: usize,
name: Option<&'static str>
) -> Result<&AlgebraicValue, InvalidFieldError>
pub fn get_field( &self, index: usize, name: Option<&'static str> ) -> Result<&AlgebraicValue, InvalidFieldError>
Borrow the value at field of self indentified by index.
The name is non-functional and is only used for error-messages.
sourcepub fn extract_field<'a, T>(
&'a self,
index: usize,
name: Option<&'static str>,
f: impl Fn(&'a AlgebraicValue) -> Option<T> + 'a
) -> Result<T, InvalidFieldError>
pub fn extract_field<'a, T>( &'a self, index: usize, name: Option<&'static str>, f: impl Fn(&'a AlgebraicValue) -> Option<T> + 'a ) -> Result<T, InvalidFieldError>
Extracts the value at field of self identified by index
and then runs it through the function f which possibly returns a T derived from value.
sourcepub fn field_as_bool(
&self,
index: usize,
named: Option<&'static str>
) -> Result<bool, InvalidFieldError>
pub fn field_as_bool( &self, index: usize, named: Option<&'static str> ) -> Result<bool, InvalidFieldError>
Interprets the value at field of self indentified by index as a bool.
sourcepub fn field_as_u8(
&self,
index: usize,
named: Option<&'static str>
) -> Result<u8, InvalidFieldError>
pub fn field_as_u8( &self, index: usize, named: Option<&'static str> ) -> Result<u8, InvalidFieldError>
Interprets the value at field of self indentified by index as a u8.
sourcepub fn field_as_u32(
&self,
index: usize,
named: Option<&'static str>
) -> Result<u32, InvalidFieldError>
pub fn field_as_u32( &self, index: usize, named: Option<&'static str> ) -> Result<u32, InvalidFieldError>
Interprets the value at field of self indentified by index as a u32.
sourcepub fn field_as_i64(
&self,
index: usize,
named: Option<&'static str>
) -> Result<i64, InvalidFieldError>
pub fn field_as_i64( &self, index: usize, named: Option<&'static str> ) -> Result<i64, InvalidFieldError>
Interprets the value at field of self indentified by index as a i64.
sourcepub fn field_as_i128(
&self,
index: usize,
named: Option<&'static str>
) -> Result<i128, InvalidFieldError>
pub fn field_as_i128( &self, index: usize, named: Option<&'static str> ) -> Result<i128, InvalidFieldError>
Interprets the value at field of self indentified by index as a i128.
sourcepub fn field_as_u128(
&self,
index: usize,
named: Option<&'static str>
) -> Result<u128, InvalidFieldError>
pub fn field_as_u128( &self, index: usize, named: Option<&'static str> ) -> Result<u128, InvalidFieldError>
Interprets the value at field of self indentified by index as a u128.
sourcepub fn field_as_str(
&self,
index: usize,
named: Option<&'static str>
) -> Result<&str, InvalidFieldError>
pub fn field_as_str( &self, index: usize, named: Option<&'static str> ) -> Result<&str, InvalidFieldError>
Interprets the value at field of self indentified by index as a string slice.
sourcepub fn field_as_bytes(
&self,
index: usize,
named: Option<&'static str>
) -> Result<&[u8], InvalidFieldError>
pub fn field_as_bytes( &self, index: usize, named: Option<&'static str> ) -> Result<&[u8], InvalidFieldError>
Interprets the value at field of self indentified by index as a byte slice.