pub enum TypeDef {
    Sum(SumType),
    Product(ProductType),
    Builtin(BuiltinType),
    Ref(AlgebraicTypeRef),
}
The SpacetimeDB Algebraic Type System (SATS) is a structural type system in which a nominal type system can be constructed.

The type system unifies the concepts sum types, product types, and built-in primitive types into a single type system.

Below are some common types you might implement in this type system.

type Unit = () // or (,) or , Product with zero elements
type Never = (|) // or | Sum with zero elements
type U8 = U8 // Builtin
type Foo = (foo: I8) != I8
type Bar = (bar: I8)
type Color = (a: I8 | b: I8) // Sum with one element
type Age = (age: U8) // Product with one element
type Option<T> = (some: T | none: ())
type Ref = &0

type AlgebraicType = (sum: SumType | product: ProductType | builtin: BuiltinType | set: AlgebraicType)
type Catalog<T> = (name: String, indices: Set<Set<Tag>>, relation: Set<>)
type CatalogEntry = { name: string, indexes: {some type}, relation: Relation }
type ElementValue = (tag: Tag, value: AlgebraicValue)
type AlgebraicValue = (sum: ElementValue | product: {ElementValue} | builtin: BuiltinValue | set: {AlgebraicValue})
type Any = (value: Bytes, type: AlgebraicType)

type Table<Row: ProductType> = (
    rows: Array<Row>
)

type HashSet<T> = (
    array: Array<T>
)

type BTreeSet<T> = (
    array: Array<T>
)

type TableType<Row: ProductType> = (
    relation: Table<Row>,
    indexes: Array<(index_type: String)>,
)

Variants

Implementations

source

impl AlgebraicType

source

pub fn as_sum_mut(&mut self) -> Option<&mut SumType>

Optionally returns mutable references to the inner fields if this is a AlgebraicType::Sum, otherwise None

source

pub fn as_sum(&self) -> Option<&SumType>

Optionally returns references to the inner fields if this is a AlgebraicType::Sum, otherwise None

source

pub fn into_sum(self) -> Result<SumType, AlgebraicType>

Returns the inner fields if this is a AlgebraicType::Sum, otherwise returns back the enum in the Err case of the result

source

pub fn as_product_mut(&mut self) -> Option<&mut ProductType>

Optionally returns mutable references to the inner fields if this is a AlgebraicType::Product, otherwise None

source

pub fn as_product(&self) -> Option<&ProductType>

Optionally returns references to the inner fields if this is a AlgebraicType::Product, otherwise None

source

pub fn into_product(self) -> Result<ProductType, AlgebraicType>

Returns the inner fields if this is a AlgebraicType::Product, otherwise returns back the enum in the Err case of the result

source

pub fn as_builtin_mut(&mut self) -> Option<&mut BuiltinType>

Optionally returns mutable references to the inner fields if this is a AlgebraicType::Builtin, otherwise None

source

pub fn as_builtin(&self) -> Option<&BuiltinType>

Optionally returns references to the inner fields if this is a AlgebraicType::Builtin, otherwise None

source

pub fn into_builtin(self) -> Result<BuiltinType, AlgebraicType>

Returns the inner fields if this is a AlgebraicType::Builtin, otherwise returns back the enum in the Err case of the result

source

pub fn as_ref_mut(&mut self) -> Option<&mut AlgebraicTypeRef>

Optionally returns mutable references to the inner fields if this is a AlgebraicType::Ref, otherwise None

source

pub fn as_ref(&self) -> Option<&AlgebraicTypeRef>

Optionally returns references to the inner fields if this is a AlgebraicType::Ref, otherwise None

source

pub fn into_ref(self) -> Result<AlgebraicTypeRef, AlgebraicType>

Returns the inner fields if this is a AlgebraicType::Ref, otherwise returns back the enum in the Err case of the result

source

impl AlgebraicType

source

pub const Bool: AlgebraicType = AlgebraicType::Builtin(BuiltinType::Bool)

source

pub const I8: AlgebraicType = AlgebraicType::Builtin(BuiltinType::I8)

source

pub const U8: AlgebraicType = AlgebraicType::Builtin(BuiltinType::U8)

source

pub const I16: AlgebraicType = AlgebraicType::Builtin(BuiltinType::I16)

source

pub const U16: AlgebraicType = AlgebraicType::Builtin(BuiltinType::U16)

source

pub const I32: AlgebraicType = AlgebraicType::Builtin(BuiltinType::I32)

source

pub const U32: AlgebraicType = AlgebraicType::Builtin(BuiltinType::U32)

source

pub const I64: AlgebraicType = AlgebraicType::Builtin(BuiltinType::I64)

source

pub const U64: AlgebraicType = AlgebraicType::Builtin(BuiltinType::U64)

source

pub const I128: AlgebraicType = AlgebraicType::Builtin(BuiltinType::I128)

source

pub const U128: AlgebraicType = AlgebraicType::Builtin(BuiltinType::U128)

source

pub const F32: AlgebraicType = AlgebraicType::Builtin(BuiltinType::F32)

source

pub const F64: AlgebraicType = AlgebraicType::Builtin(BuiltinType::F64)

source

pub const String: AlgebraicType = AlgebraicType::Builtin(BuiltinType::String)

source

pub fn bytes() -> AlgebraicType

source

impl AlgebraicType

source

pub fn make_meta_type() -> AlgebraicType

This is a static function that constructs the type of AlgebraicType and returns it as an AlgebraicType. This could alternatively be implemented as a regular AlgebraicValue or as a static variable.

source

pub fn make_never_type() -> AlgebraicType

source

pub const UNIT_TYPE: AlgebraicType = AlgebraicType::Product(ProductType{ elements: Vec::new(),})

source

pub fn make_option_type(some_type: AlgebraicType) -> AlgebraicType

source

pub fn as_value(&self) -> AlgebraicValue

source

pub fn from_value( value: &AlgebraicValue ) -> Result<AlgebraicType, ValueDeserializeError>

source

impl AlgebraicType

source

pub fn decode<'a>( bytes: &mut impl BufReader<'a> ) -> Result<AlgebraicType, DecodeError>

source

pub fn encode(&self, bytes: &mut impl BufWriter)