pub trait ArrayAccess<'de> {
type Element;
type Error: Error;
// Required method
fn next_element(&mut self) -> Result<Option<Self::Element>, Self::Error>;
// Provided methods
fn validate_next_element(&mut self) -> Result<Option<()>, Self::Error> { ... }
fn size_hint(&self) -> Option<usize> { ... }
}Expand description
Provides an ArrayVisitor with access to each element of the array in the input.
This is a trait that a Deserializer passes to an ArrayVisitor implementation.