Enum spacetimedb_lib::AlgebraicValue
source · pub enum AlgebraicValue {
Sum(SumValue),
Product(ProductValue),
Builtin(BuiltinValue),
}Expand description
A value in SATS typed at some AlgebraicType.
Values are type erased, so they do not store their type. This is important mainly for space efficiency, including network latency and bandwidth.
These are only values and not expressions.
That is, they are canonical and cannot be simplified further by some evaluation.
So forms like 42 + 24 are not represented in an AlgebraicValue.
Variants
Sum(SumValue)
A structural sum value.
Given a sum type { N_0(T_0), N_1(T_1), ..., N_n(T_n) }
where N_i denotes a variant name
and where T_i denotes the type the variant stores,
a sum value makes a specific choice as to the variant.
So for example, we might chose N_1(T_1)
and represent this choice with (1, v) where v is a value of type T_1.
Product(ProductValue)
A structural product value.
Given a product type { N_0: T_0, N_1: T_1, ..., N_n: T_n }
where N_i denotes a field / element name
and where T_i denotes the type the field stores,
a product value stores a value v_i of type T_i for each field N_i.
Builtin(BuiltinValue)
A builtin value that has a builtin type.
Implementations
sourceimpl AlgebraicValue
impl AlgebraicValue
sourcepub fn as_sum_mut(&mut self) -> Option<&mut SumValue>
pub fn as_sum_mut(&mut self) -> Option<&mut SumValue>
Optionally returns mutable references to the inner fields if this is a AlgebraicValue::Sum, otherwise None
sourcepub fn as_sum(&self) -> Option<&SumValue>
pub fn as_sum(&self) -> Option<&SumValue>
Optionally returns references to the inner fields if this is a AlgebraicValue::Sum, otherwise None
sourcepub fn into_sum(self) -> Result<SumValue, AlgebraicValue>
pub fn into_sum(self) -> Result<SumValue, AlgebraicValue>
Returns the inner fields if this is a AlgebraicValue::Sum, otherwise returns back the enum in the Err case of the result
sourcepub fn is_product(&self) -> bool
pub fn is_product(&self) -> bool
Returns true if this is a AlgebraicValue::Product, otherwise false
sourcepub fn as_product_mut(&mut self) -> Option<&mut ProductValue>
pub fn as_product_mut(&mut self) -> Option<&mut ProductValue>
Optionally returns mutable references to the inner fields if this is a AlgebraicValue::Product, otherwise None
sourcepub fn as_product(&self) -> Option<&ProductValue>
pub fn as_product(&self) -> Option<&ProductValue>
Optionally returns references to the inner fields if this is a AlgebraicValue::Product, otherwise None
sourcepub fn into_product(self) -> Result<ProductValue, AlgebraicValue>
pub fn into_product(self) -> Result<ProductValue, AlgebraicValue>
Returns the inner fields if this is a AlgebraicValue::Product, otherwise returns back the enum in the Err case of the result
sourcepub fn is_builtin(&self) -> bool
pub fn is_builtin(&self) -> bool
Returns true if this is a AlgebraicValue::Builtin, otherwise false
sourcepub fn as_builtin_mut(&mut self) -> Option<&mut BuiltinValue>
pub fn as_builtin_mut(&mut self) -> Option<&mut BuiltinValue>
Optionally returns mutable references to the inner fields if this is a AlgebraicValue::Builtin, otherwise None
sourcepub fn as_builtin(&self) -> Option<&BuiltinValue>
pub fn as_builtin(&self) -> Option<&BuiltinValue>
Optionally returns references to the inner fields if this is a AlgebraicValue::Builtin, otherwise None
sourcepub fn into_builtin(self) -> Result<BuiltinValue, AlgebraicValue>
pub fn into_builtin(self) -> Result<BuiltinValue, AlgebraicValue>
Returns the inner fields if this is a AlgebraicValue::Builtin, otherwise returns back the enum in the Err case of the result
sourceimpl AlgebraicValue
impl AlgebraicValue
sourcepub const UNIT: AlgebraicValue = Self::product(Vec::new())
pub const UNIT: AlgebraicValue = Self::product(Vec::new())
The canonical unit value defined as the nullary product value ().
The type of UNIT is ().
sourcepub fn as_bool(&self) -> Option<&bool>
pub fn as_bool(&self) -> Option<&bool>
Interpret the value as a bool or None if it isn’t a bool value.
sourcepub fn as_i8(&self) -> Option<&i8>
pub fn as_i8(&self) -> Option<&i8>
Interpret the value as an i8 or None if it isn’t a i8 value.
sourcepub fn as_i16(&self) -> Option<&i16>
pub fn as_i16(&self) -> Option<&i16>
Interpret the value as an i16 or None if it isn’t an i16 value.
sourcepub fn as_u16(&self) -> Option<&u16>
pub fn as_u16(&self) -> Option<&u16>
Interpret the value as a u16 or None if it isn’t a u16 value.
sourcepub fn as_i32(&self) -> Option<&i32>
pub fn as_i32(&self) -> Option<&i32>
Interpret the value as an i32 or None if it isn’t an i32 value.
sourcepub fn as_u32(&self) -> Option<&u32>
pub fn as_u32(&self) -> Option<&u32>
Interpret the value as a u32 or None if it isn’t a u32 value.
sourcepub fn as_i64(&self) -> Option<&i64>
pub fn as_i64(&self) -> Option<&i64>
Interpret the value as an i64 or None if it isn’t an i64 value.
sourcepub fn as_u64(&self) -> Option<&u64>
pub fn as_u64(&self) -> Option<&u64>
Interpret the value as a u64 or None if it isn’t a u64 value.
sourcepub fn as_i128(&self) -> Option<&i128>
pub fn as_i128(&self) -> Option<&i128>
Interpret the value as an i128 or None if it isn’t an i128 value.
sourcepub fn as_u128(&self) -> Option<&u128>
pub fn as_u128(&self) -> Option<&u128>
Interpret the value as a u128 or None if it isn’t a u128 value.
sourcepub fn as_f32(&self) -> Option<&ConstrainedFloat<f32, UnitConstraint<f32>>>
pub fn as_f32(&self) -> Option<&ConstrainedFloat<f32, UnitConstraint<f32>>>
Interpret the value as a f32 or None if it isn’t a f32 value.
sourcepub fn as_f64(&self) -> Option<&ConstrainedFloat<f64, UnitConstraint<f64>>>
pub fn as_f64(&self) -> Option<&ConstrainedFloat<f64, UnitConstraint<f64>>>
Interpret the value as a f64 or None if it isn’t a f64 value.
sourcepub fn as_string(&self) -> Option<&String>
pub fn as_string(&self) -> Option<&String>
Interpret the value as a String or None if it isn’t a String value.
sourcepub fn as_bytes(&self) -> Option<&Vec<u8, Global>>
pub fn as_bytes(&self) -> Option<&Vec<u8, Global>>
Interpret the value as a Vec<u8> or None if it isn’t a Vec<u8> value.
sourcepub fn as_array(&self) -> Option<&ArrayValue>
pub fn as_array(&self) -> Option<&ArrayValue>
Interpret the value as an ArrayValue or None if it isn’t an ArrayValue value.
sourcepub fn as_map(
&self
) -> Option<&BTreeMap<AlgebraicValue, AlgebraicValue, Global>>
pub fn as_map( &self ) -> Option<&BTreeMap<AlgebraicValue, AlgebraicValue, Global>>
Interpret the value as a map or None if it isn’t a map value.
sourcepub fn into_bool(self) -> Result<bool, AlgebraicValue>
pub fn into_bool(self) -> Result<bool, AlgebraicValue>
Convert the value into a bool or Err(self) if it isn’t a bool value.
sourcepub fn into_i8(self) -> Result<i8, AlgebraicValue>
pub fn into_i8(self) -> Result<i8, AlgebraicValue>
Convert the value into an i8 or Err(self) if it isn’t an i8 value.
sourcepub fn into_u8(self) -> Result<u8, AlgebraicValue>
pub fn into_u8(self) -> Result<u8, AlgebraicValue>
Convert the value into a u8 or Err(self) if it isn’t a u8 value.
sourcepub fn into_i16(self) -> Result<i16, AlgebraicValue>
pub fn into_i16(self) -> Result<i16, AlgebraicValue>
Convert the value into an i16 or Err(self) if it isn’t an i16 value.
sourcepub fn into_u16(self) -> Result<u16, AlgebraicValue>
pub fn into_u16(self) -> Result<u16, AlgebraicValue>
Convert the value into a u16 or Err(self) if it isn’t a u16 value.
sourcepub fn into_i32(self) -> Result<i32, AlgebraicValue>
pub fn into_i32(self) -> Result<i32, AlgebraicValue>
Convert the value into an i32 or Err(self) if it isn’t an i32 value.
sourcepub fn into_u32(self) -> Result<u32, AlgebraicValue>
pub fn into_u32(self) -> Result<u32, AlgebraicValue>
Convert the value into a u32 or Err(self) if it isn’t a u32 value.
sourcepub fn into_i64(self) -> Result<i64, AlgebraicValue>
pub fn into_i64(self) -> Result<i64, AlgebraicValue>
Convert the value into an i64 or Err(self) if it isn’t an i64 value.
sourcepub fn into_u64(self) -> Result<u64, AlgebraicValue>
pub fn into_u64(self) -> Result<u64, AlgebraicValue>
Convert the value into a u64 or Err(self) if it isn’t a u64 value.
sourcepub fn into_i128(self) -> Result<i128, AlgebraicValue>
pub fn into_i128(self) -> Result<i128, AlgebraicValue>
Convert the value into an i128 or Err(self) if it isn’t an i128 value.
sourcepub fn into_u128(self) -> Result<u128, AlgebraicValue>
pub fn into_u128(self) -> Result<u128, AlgebraicValue>
Convert the value into a u128 or Err(self) if it isn’t a u128 value.
sourcepub fn into_f32(
self
) -> Result<ConstrainedFloat<f32, UnitConstraint<f32>>, AlgebraicValue>
pub fn into_f32( self ) -> Result<ConstrainedFloat<f32, UnitConstraint<f32>>, AlgebraicValue>
Convert the value into a f32 or Err(self) if it isn’t a f32 value.
sourcepub fn into_f64(
self
) -> Result<ConstrainedFloat<f64, UnitConstraint<f64>>, AlgebraicValue>
pub fn into_f64( self ) -> Result<ConstrainedFloat<f64, UnitConstraint<f64>>, AlgebraicValue>
Convert the value into a f64 or Err(self) if it isn’t a f64 value.
sourcepub fn into_string(self) -> Result<String, AlgebraicValue>
pub fn into_string(self) -> Result<String, AlgebraicValue>
Convert the value into a String or Err(self) if it isn’t a String value.
sourcepub fn into_bytes(self) -> Result<Vec<u8, Global>, AlgebraicValue>
pub fn into_bytes(self) -> Result<Vec<u8, Global>, AlgebraicValue>
Convert the value into a Vec<u8> or Err(self) if it isn’t a Vec<u8> value.
sourcepub fn into_array(self) -> Result<ArrayValue, AlgebraicValue>
pub fn into_array(self) -> Result<ArrayValue, AlgebraicValue>
Convert the value into an ArrayValue or Err(self) if it isn’t an ArrayValue value.
sourcepub fn into_map(
self
) -> Result<BTreeMap<AlgebraicValue, AlgebraicValue, Global>, AlgebraicValue>
pub fn into_map( self ) -> Result<BTreeMap<AlgebraicValue, AlgebraicValue, Global>, AlgebraicValue>
Convert the value into a map or Err(self) if it isn’t a map value.
sourcepub const fn Bool(v: bool) -> AlgebraicValue
pub const fn Bool(v: bool) -> AlgebraicValue
Returns an AlgebraicValue representing v: bool.
sourcepub const fn I8(v: i8) -> AlgebraicValue
pub const fn I8(v: i8) -> AlgebraicValue
Returns an AlgebraicValue representing v: i8.
sourcepub const fn U8(v: u8) -> AlgebraicValue
pub const fn U8(v: u8) -> AlgebraicValue
Returns an AlgebraicValue representing v: u8.
sourcepub const fn I16(v: i16) -> AlgebraicValue
pub const fn I16(v: i16) -> AlgebraicValue
Returns an AlgebraicValue representing v: i16.
sourcepub const fn U16(v: u16) -> AlgebraicValue
pub const fn U16(v: u16) -> AlgebraicValue
Returns an AlgebraicValue representing v: u16.
sourcepub const fn I32(v: i32) -> AlgebraicValue
pub const fn I32(v: i32) -> AlgebraicValue
Returns an AlgebraicValue representing v: i32.
sourcepub const fn U32(v: u32) -> AlgebraicValue
pub const fn U32(v: u32) -> AlgebraicValue
Returns an AlgebraicValue representing v: u32.
sourcepub const fn I64(v: i64) -> AlgebraicValue
pub const fn I64(v: i64) -> AlgebraicValue
Returns an AlgebraicValue representing v: i64.
sourcepub const fn U64(v: u64) -> AlgebraicValue
pub const fn U64(v: u64) -> AlgebraicValue
Returns an AlgebraicValue representing v: u64.
sourcepub const fn I128(v: i128) -> AlgebraicValue
pub const fn I128(v: i128) -> AlgebraicValue
Returns an AlgebraicValue representing v: i128.
sourcepub const fn U128(v: u128) -> AlgebraicValue
pub const fn U128(v: u128) -> AlgebraicValue
Returns an AlgebraicValue representing v: u128.
sourcepub const fn F32(
v: ConstrainedFloat<f32, UnitConstraint<f32>>
) -> AlgebraicValue
pub const fn F32( v: ConstrainedFloat<f32, UnitConstraint<f32>> ) -> AlgebraicValue
Returns an AlgebraicValue representing v: f32.
sourcepub const fn F64(
v: ConstrainedFloat<f64, UnitConstraint<f64>>
) -> AlgebraicValue
pub const fn F64( v: ConstrainedFloat<f64, UnitConstraint<f64>> ) -> AlgebraicValue
Returns an AlgebraicValue representing v: f64.
sourcepub const fn String(v: String) -> AlgebraicValue
pub const fn String(v: String) -> AlgebraicValue
Returns an AlgebraicValue representing v: String.
sourcepub const fn Bytes(v: Vec<u8, Global>) -> AlgebraicValue
pub const fn Bytes(v: Vec<u8, Global>) -> AlgebraicValue
Returns an AlgebraicValue representing v: Vec<u8>.
sourcepub fn ArrayOf(val: impl Into<ArrayValue>) -> AlgebraicValue
pub fn ArrayOf(val: impl Into<ArrayValue>) -> AlgebraicValue
Returns an AlgebraicValue for a val which can be converted into an ArrayValue.
sourcepub fn OptionSome(v: AlgebraicValue) -> AlgebraicValue
pub fn OptionSome(v: AlgebraicValue) -> AlgebraicValue
Returns an AlgebraicValue for some: v.
The some variant is assigned the tag 0.
sourcepub fn OptionNone() -> AlgebraicValue
pub fn OptionNone() -> AlgebraicValue
Returns an AlgebraicValue for none.
The none variant is assigned the tag 1.
sourcepub fn sum(tag: u8, value: AlgebraicValue) -> AlgebraicValue
pub fn sum(tag: u8, value: AlgebraicValue) -> AlgebraicValue
Returns an AlgebraicValue representing a sum value with tag and value.
sourcepub const fn product(elements: Vec<AlgebraicValue, Global>) -> AlgebraicValue
pub const fn product(elements: Vec<AlgebraicValue, Global>) -> AlgebraicValue
Returns an AlgebraicValue representing a product value with the given elements.
sourcepub const fn map(
map: BTreeMap<AlgebraicValue, AlgebraicValue, Global>
) -> AlgebraicValue
pub const fn map( map: BTreeMap<AlgebraicValue, AlgebraicValue, Global> ) -> AlgebraicValue
Returns an AlgebraicValue representing a map value defined by the given map.
sourcepub fn type_of(&self) -> AlgebraicType
pub fn type_of(&self) -> AlgebraicType
Infer the AlgebraicType of an AlgebraicValue.
sourceimpl AlgebraicValue
impl AlgebraicValue
pub fn decode<'a>( algebraic_type: &<AlgebraicValue as Value>::Type, bytes: &mut impl BufReader<'a> ) -> Result<AlgebraicValue, DecodeError>
pub fn encode(&self, bytes: &mut impl BufWriter)