pub enum ScheduleAt {
Interval(TimeDuration),
Time(Timestamp),
}Expand description
When a scheduled reducer should execute, either at a specific point in time, or at regular intervals for repeating schedules.
Stored in reducer-scheduling tables as a column.
This is a special type.
Variants
Interval(TimeDuration)
A regular interval at which the repeated reducer is scheduled.
Value is a TimeDuration, which has nanosecond precision.
Time(Timestamp)
A specific time to which the reducer is scheduled.
Implementations
Sourceimpl ScheduleAt
impl ScheduleAt
Sourcepub fn to_duration_from(&self, from: Timestamp) -> Duration
pub fn to_duration_from(&self, from: Timestamp) -> Duration
Converts the ScheduleAt to a std::time::Duration from now.
Returns std::time::Duration::ZERO if self represents a time in the past.
Sourcepub fn to_timestamp_from(&self, from: Timestamp) -> Timestamp
pub fn to_timestamp_from(&self, from: Timestamp) -> Timestamp
Converts the ScheduleAt to a Timestamp.
If self is an interval, returns from + dur.
Sourcepub fn get_type() -> AlgebraicType
pub fn get_type() -> AlgebraicType
Get the special AlgebraicType for ScheduleAt.